/*
	eth_recv - Un programa sencillo para recibir paquetes ethernet raw en Linux
	(soporta modo promiscuo)
	Copyright (C) 2005  Avelino Herrera Morales

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <linux/if_packet.h>
#include <linux/if_ether.h>
#include <linux/if_arp.h>

/* #define  SET_PROMISCUOUS  1 */

/* #define  ETH_FRAME_LEN  1518 */

char device_name[80] = "eth0";
char received_string[80] = "";
int s;
struct sockaddr_ll sa;
struct ifreq ifr;
int ifindex;
unsigned char *buffer;
unsigned char *ether_head;
unsigned char *data;
struct ethhdr *eh;
int send_result = 0;

/* mi direccin MAC */
unsigned char my_mac[ETH_ALEN];
/* direccin MAC de origen */
unsigned char src_mac[ETH_ALEN];
/* direccin MAC de origen permitida */
unsigned char allowed_src_mac[ETH_ALEN];
/* direccin MAC de destino (til slo para el modo promiscuo) */
unsigned char dest_mac[ETH_ALEN];

int filter = 0;


void set_printable(char *v) {
	while (*v != 0) {
		if (!isgraph(*v))
			*v = '.';
		v++;
	}
}

int main(int argc, char *argv[]) {
	int i;

	if (argc < 2) {
		fprintf(stderr, "usage: %s interface [al:lo:we:dh:wa:dd]\n", argv[0]);
		exit(-1);
	}
	strcpy(device_name, argv[1]);
	if (argc > 2) {
		sscanf(argv[2], "%x:%x:%x:%x:%x:%x", &allowed_src_mac[0], &allowed_src_mac[1], &allowed_src_mac[2], &allowed_src_mac[3], &allowed_src_mac[4], &allowed_src_mac[5]);
		filter = 1;
		printf("show only packets from %02x:%02x:%02x:%02x:%02x:%02x\n", allowed_src_mac[0], allowed_src_mac[1], allowed_src_mac[2], allowed_src_mac[3], allowed_src_mac[4], allowed_src_mac[5]);
	}
	else {
		printf("show all incoming packets\n");
		filter = 0;
	}
	if ((s = socket(AF_PACKET, SOCK_RAW, htons(ETH_P_ALL))) < 0) {
		perror("socket");
		exit(-1);
	}
#ifdef  SET_PROMISCUOUS
	/* ponemos, si procede, el interface de red en modo promiscuo */
	strncpy(ifr.ifr_name, device_name, IFNAMSIZ);
	if (ioctl(s, SIOCGIFFLAGS, &ifr) == -1) {
		perror("SIOCGIFFLAGS");
		exit(1);
	}
	ifr.ifr_flags |= IFF_PROMISC;
	if (ioctl(s, SIOCSIFFLAGS, &ifr) == -1) {
		perror("SIOCSIFFLAGS");
		exit(1);
	}
	printf("set promiscuous mode\n");
#endif
	/* obtenemos el ndice del interface */
	strncpy(ifr.ifr_name, device_name, IFNAMSIZ);
	if (ioctl(s, SIOCGIFINDEX, &ifr) == -1) {
		perror("SIOCGIFINDEX");
		exit(1);
	}
	ifindex = ifr.ifr_ifindex;
	printf("got interface %s index %i\n", device_name, ifindex);
	/* obtenemos la direccin hardware */
	strncpy(ifr.ifr_name, device_name, IFNAMSIZ);
	if (ioctl(s, SIOCGIFHWADDR, &ifr) == -1) {
		perror("SIOCGIFINDEX");
		exit(1);
	}
	printf("got interface %s hardware address ", device_name, ETH_ALEN);
	for (i = 0; i < ETH_ALEN; i++) {
		printf("%02x", (unsigned char) ifr.ifr_hwaddr.sa_data[i]);
		if (i < 5)
			printf(":");
	}
	printf("\n");
	memcpy(my_mac, ifr.ifr_hwaddr.sa_data, ETH_ALEN);
	/* configuracin del socket */
	buffer = (unsigned char *) malloc(ETH_FRAME_LEN);
	while (1) {
		int length = recvfrom(s, buffer, ETH_FRAME_LEN, 0, NULL, NULL);
		if (length < 0) {
			perror("recvfrom");
			break;
		}
		memcpy(dest_mac, buffer, ETH_ALEN);
		memcpy(src_mac, buffer + ETH_ALEN, ETH_ALEN);
		if (filter) {
			if (memcmp(src_mac, allowed_src_mac, ETH_ALEN) != 0)
				continue;
		}
		i = ntohs(*((unsigned short int *) (buffer + ETH_ALEN + ETH_ALEN)));
		if (i > 79)
			i = 79;
		strncpy(received_string, buffer + ETH_ALEN + ETH_ALEN + 2, i);
		received_string[i] = 0;
		set_printable(received_string);
		printf("packet from %02x:%02x:%02x:%02x:%02x:%02x to %02x:%02x:%02x:%02x:%02x:%02x at %s: \"%s\"\n", src_mac[0], src_mac[1], src_mac[2], src_mac[3], src_mac[4], src_mac[5], dest_mac[0], dest_mac[1], dest_mac[2], dest_mac[3], dest_mac[4], dest_mac[5], device_name, received_string);
	}
	close(s);
	return 0;
}
