/*
	eth_send - Un programa sencillo para transmitir paquetes ethernet raw en Linux
	Copyright (C) 2005  Avelino Herrera Morales

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <linux/if_packet.h>
#include <linux/if_ether.h>
#include <linux/if_arp.h>

/* #define  ETH_FRAME_LEN  1518 */

char device_name[80] = "eth0";
char string_to_send[80] = "";
int s;
struct sockaddr_ll sa;
struct ifreq ifr;
int ifindex;
unsigned char *buffer;
unsigned char *ether_head;
unsigned char *data;
struct ethhdr *eh;
int send_result = 0;
int prompt = 0;

/* mi direccin MAC */
unsigned char src_mac[ETH_ALEN];
/* direccin MAC destino (por defecto broadcast) */
unsigned char dest_mac[ETH_ALEN] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};


int main(int argc, char *argv[]) {
	int i;

	if (argc < 3) {
		fprintf(stderr, "Usage: %s interface ha:rd:wa:re:ad:dr [\"message to send\"]\n", argv[0]);
		exit(-1);
	}
	/* get parameters */
	strcpy(device_name, argv[1]);
	sscanf(argv[2], "%x:%x:%x:%x:%x:%x", &dest_mac[0], &dest_mac[1], &dest_mac[2], &dest_mac[3], &dest_mac[4], &dest_mac[5]);
	if (argc > 3) {
		strcpy(string_to_send, argv[3]);
		prompt = 0;
	}
	else
		prompt = 1;
	printf("sending \"%s\" to %02x:%02x:%02x:%02x:%02x:%02x using %s\n", string_to_send, dest_mac[0], dest_mac[1], dest_mac[2], dest_mac[3], dest_mac[4], dest_mac[5], device_name);
	if ((s = socket(AF_PACKET, SOCK_RAW, htons(ETH_P_ALL))) < 0) {
		perror("socket");
		exit(-1);
	}
	/* obtenemos el ndice del interface */
	strncpy(ifr.ifr_name, device_name, IFNAMSIZ);
	if (ioctl(s, SIOCGIFINDEX, &ifr) == -1) {
		perror("SIOCGIFINDEX");
		exit(1);
	}
	ifindex = ifr.ifr_ifindex;
	printf("got interface %s index %i\n", device_name, ifindex);
	/* obtenemos la direccin hardware */
	strncpy(ifr.ifr_name, device_name, IFNAMSIZ);
	if (ioctl(s, SIOCGIFHWADDR, &ifr) == -1) {
		perror("SIOCGIFINDEX");
		exit(1);
	}
	printf("got interface %s hardware address ", device_name, ETH_ALEN);
	for (i = 0; i < ETH_ALEN; i++) {
		printf("%02x", (unsigned char) ifr.ifr_hwaddr.sa_data[i]);
		if (i < 5)
			printf(":");
	}
	printf("\n");
	memcpy(src_mac, ifr.ifr_hwaddr.sa_data, ETH_ALEN);
	/* configuracin del socket */
	buffer = (unsigned char *) malloc(ETH_FRAME_LEN);
	ether_head = buffer;
	data = buffer + 14;
	eh = (struct ethhdr *) ether_head;
	sa.sll_family = PF_PACKET;
	sa.sll_protocol = htons(ETH_P_IP);
	sa.sll_ifindex = ifindex;                   /* interface index */
	sa.sll_hatype = ARPHRD_ETHER;
	sa.sll_pkttype = PACKET_OTHERHOST;
	sa.sll_halen = ETH_ALEN;              /* tamao de la direccin ethernet */
	bzero(sa.sll_addr, 8);
	memcpy(sa.sll_addr, dest_mac, ETH_ALEN);
	/* el paquete ethernet */
	memcpy(buffer, dest_mac, ETH_ALEN);
	memcpy(buffer + ETH_ALEN, src_mac, ETH_ALEN);
	if (prompt) {
		while (1) {
			bzero(string_to_send, 80);
			fgets(string_to_send, 79, stdin);
			if (strncmp(string_to_send, "exit", 4) == 0)
				break;
			i = strlen(string_to_send);
			if (i < 48)
				i = 68;
			eh->h_proto = htons(i);
			bzero(data, 1500);
			strcpy(data, string_to_send);
			/* enviamos el paquete */
			send_result = sendto(s, buffer, ETH_FRAME_LEN, 0, (struct sockaddr *) &sa, sizeof(sa));
			if (send_result < 0) {
				perror("sendto");
				exit(-1);
			}
			printf("Raw ethernet packet sent (send_result = %d).\n", send_result);
		}
	}
	else {
		i = strlen(string_to_send);
		if (i < 48)
			i = 68;
		eh->h_proto = htons(i);
		bzero(data, 1500);
		strcpy(data, string_to_send);
		/* enviamos el paquete */
		send_result = sendto(s, buffer, ETH_FRAME_LEN, 0, (struct sockaddr *) &sa, sizeof(sa));
		if (send_result < 0) {
			perror("sendto");
			exit(-1);
		}
		printf("Raw ethernet packet sent (send_result = %d).\n", send_result);
	}
	close(s);
	return 0;
}
